/* 
 * Copyright 2022,2024 NXP
 *
 * SPDX-License-Identifier: Apache-2.0
 * 
 */

#include <iot_agent_psa_sign_test.h>
#include <psa/crypto.h>
#include <nxp_iot_agent_macros.h>

uint8_t psa_import_blob_buffer[] =
{
  0x40, 0x0B, 0x65, 0x64, 0x67, 0x65, 0x6C, 0x6F, 0x63, 0x6B, 0x32, 0x67, 0x6F,
  0x41, 0x04, 0x00, 0x00, 0x80, 0x00, 0x42, 0x04, 0x06, 0x00, 0x06, 0x09, 0x43,
  0x04, 0x00, 0x00, 0x14, 0x00, 0x44, 0x02, 0x71, 0x12, 0x45, 0x04, 0x00, 0x00,
  0x01, 0x00, 0x46, 0x04, 0x80, 0x00, 0x01, 0x01, 0x50, 0x04, 0x3F, 0xFF, 0x02,
  0x10, 0x51, 0x04, 0x00, 0x00, 0x00, 0x01, 0x53, 0x04, 0x3F, 0xFF, 0x02, 0x12,
  0x54, 0x04, 0x00, 0x00, 0x00, 0x01, 0x55, 0x30, 0xB9, 0xC7, 0xA7, 0x72, 0xB7,
  0xB0, 0xFB, 0x4D, 0x10, 0x5E, 0x64, 0xA9, 0xD5, 0x27, 0x5D, 0x45, 0xBC, 0xBE,
  0xC5, 0x65, 0xFE, 0x13, 0x2F, 0xCC, 0x63, 0xC6, 0x0A, 0xAE, 0xCC, 0x9F, 0x65,
  0x5D, 0x2F, 0x9D, 0x1E, 0x1B, 0x7C, 0xAF, 0xD5, 0x35, 0x88, 0x83, 0x27, 0x30,
  0x2A, 0xCA, 0xBD, 0x51, 0x5E, 0x10, 0xCE, 0x24, 0x09, 0x4D, 0xCA, 0x5A, 0x01,
  0xD3, 0x85, 0x35, 0x5E, 0xEC, 0xDC, 0x26, 0x88, 0xAB
};

uint8_t public_key[] =
{
  0x04, 0x94, 0x17, 0x68, 0xCC, 0x2D, 0x8F, 0xB6, 0x62, 0x11, 0x6C, 0xC3, 0xBD,
  0x6C, 0x62, 0x99, 0x48, 0x48, 0xD8, 0xF3, 0x3B, 0xAE, 0xCA, 0x7B, 0xCA, 0xCC,
  0x26, 0xC7, 0xCC, 0xE9, 0x1B, 0x24, 0x01, 0xED, 0xE4, 0x97, 0x8C, 0xE1, 0xCB,
  0x09, 0x7A, 0x69, 0xB3, 0x83, 0x39, 0x01, 0x0A, 0x53, 0xC8, 0xE2, 0x71, 0xC0,
  0x44, 0xC0, 0x31, 0xB7, 0xEE, 0x6A, 0xBA, 0xFA, 0x80, 0x03, 0x2D, 0x9B, 0xAD
};

static uint8_t hash[] =
{
  0xA6, 0x67, 0x28, 0x26, 0x75, 0xF4, 0x87, 0x60, 0x21, 0xD3, 0x92, 0xAA, 0x65,
  0x92, 0xF3, 0x9D, 0xAB, 0xF7, 0x18, 0x74, 0x8C, 0x4B, 0x73, 0x85, 0x63, 0xCB,
  0x9D, 0x5D, 0xC8, 0xF2, 0x1F, 0x24
};

iot_agent_status_t iot_agent_verify_psa_import(void)
{
  
  iot_agent_status_t agent_status = IOT_AGENT_SUCCESS;
  psa_status_t psa_status;
  
  uint8_t signature[PSA_SIGNATURE_MAX_SIZE];
  size_t signature_length;
  
  // Sign Hash
  psa_status = psa_sign_hash(IMPORT_OBJ_ID, // must match with key id of provisioned blob
                             PSA_ALG_ECDSA(PSA_ALG_SHA_256),
                             hash,
                             sizeof(hash),
                             signature,
                             PSA_SIGNATURE_MAX_SIZE,
                             &signature_length);
  PSA_SUCCESS_OR_EXIT_MSG("psa_sign_hash failed with provisioned key: 0x%x", psa_status);
  
exit:
  return agent_status;
}

iot_agent_status_t iot_agent_verify_psa_export(void)
{
  uint8_t output_buffer[PSA_EXPORT_KEY_PAIR_MAX_SIZE];
  size_t data_length;
  
  iot_agent_status_t agent_status = IOT_AGENT_SUCCESS;
  psa_status_t psa_status;
  
  psa_status = psa_export_key(IMPORT_OBJ_ID_TMP, output_buffer, 
                              PSA_EXPORT_KEY_PAIR_MAX_SIZE,
                              &data_length);
  PSA_SUCCESS_OR_EXIT_MSG("psa_export_key failed: 0x%x", psa_status);
exit:
  return agent_status;
}